from django.shortcuts import render, redirect
from request_filters.utils import * 


class IPCheckMiddleware():

    def __init__(self, get_response):
        self.get_response = get_response

    def process_request(self, request):
        # This code is executed just before the view is called to process the requests
        return request     

    def process_response(self, request, response):
        # After the view has finished executing, this code is executed to process the response. 
        return response

    def __call__(self, request):

        # Code to be executed for each request before the view (and later middleware) are called.
        request =  self.process_request(request)

        response = self.get_response(request)

        # Code to be executed for each request/response after the view is called.
        response =  self.process_response(request, response)
        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        # This code is executed just before the view is called
        if view_func.__name__ != "_exempt_IPCheckMiddleware":
            client_ip, is_routable = get_client_ip(request)
            userIp = getUserIpInfo(client_ip)
            if isAnonymousIP(userIp):
                if setting('IP_BLOCK_VIEW') is None:
                    return HttpResponse("<h1>We can't allow your request, because you are using VPN or Proxy or Tor or Relay.</h1>", status=418)
                else:
                    view = import_string(setting('IP_BLOCK_VIEW'))
                    return view(request)

    def process_template_response(self, request, response):
        # After the view has finished executing, this code is executed if the response contains a render() method
        return response

    def process_exception(self, request, exception):
        # This code is executed if an exception is raised
        return exception


from django.conf import settings
import requests
import json
from django.http import HttpResponse
from ipware import get_client_ip
from django.utils.module_loading import import_string


def setting(name, default=None):
    """
    Helper function to get a Django setting by name. If setting doesn't exists
    it will return a default.

    :param name: Name of setting
    :type name: str
    :param default: Value if setting is not found
    :returns: Setting's value
    """
    return getattr(settings, name, default)


def getUserIpInfo(ip_address):
    url = f"https://vpnapi.io/api/{ip_address}?key={setting('VPNAPI_KEY')}"
    headers = {
        'Content-Type': "application/x-www-form-urlencoded",
        'Cache-Control': "no-cache",
    }
    ipInfo = requests.request("GET", url, headers=headers)
    ipInfo = json.loads(ipInfo.text)
    return ipInfo

def isAnonymousIP(userIp, block_vpn=setting("BLOCK_VPN", True), block_proxy=setting("BLOCK_PROXY", True), block_tor=setting("BLOCK_TOR", True), block_relay=setting("BLOCK_RELAY", True)):
    security = userIp.get('security', {})
    block_user = False
    if block_vpn and security.get('vpn') == True:
        block_user = True
    if block_proxy and security.get('proxy') == True:
        block_user = True
    if block_tor and security.get('tor') == True:
        block_user = True
    if block_relay and security.get('relay') == True:
        block_user = True
    return block_user


from request_filters.utils import * 

def exempt_IPCheckMiddleware(func):
    """ view decorator, the sole purpose to is 'rename' the function to
    '_exempt_IPCheckMiddleware'.
    So we can identify functions those name is '_exempt_IPCheckMiddleware', to skip this view function from IPCheckMiddleware"""

    def _exempt_IPCheckMiddleware(*args, **kwargs):
        return func(*args, **kwargs)

    # copiedFunc = func
    # copiedFunc.__name__ = '_exempt_IPCheckMiddleware'
    return _exempt_IPCheckMiddleware


def prevent_anonymous_ip(func, block_vpn=True, block_proxy=True, block_tor=True, block_relay=True):
    def innerFunction(*args, **kwargs):
        request = args[0]
        client_ip, is_routable = get_client_ip(request)
        userIp = getUserIpInfo(client_ip)
        if isAnonymousIP(userIp, block_vpn, block_proxy, block_tor, block_relay):
            if setting('IP_BLOCK_VIEW') is None:
                return HttpResponse("<h1>We can't allow your request, because you are using VPN or Proxy or Tor or Relay.</h1>", status=418)
            else:
                view = import_string(setting('IP_BLOCK_VIEW'))
                return view(request)
        else:
            return func(*args, **kwargs)
    return innerFunction